from dataclasses import dataclass
from minieval.datatypes import Instance, LMOutput, Scorer
from minieval.execute.local_python import check_correctness
from minieval.score.core import ScoreType


@dataclass
class CodeExecution(Scorer):
    """Query a code executor with unit tests. Return the result of the unit tests"""
    
    timeout: int = 20
    name = "pass"
    _type = ScoreType.BINARY

    def _score_response_single(self, input: Instance, output: LMOutput) -> LMOutput:
        code_completion_request = output.extracted_answer

        assert isinstance(
            code_completion_request, dict
        ), f'Code completion must be a dict: {code_completion_request}'

        result = check_correctness(
            res_id=input.metadata["id"],
            completion=code_completion_request["completion"],
            test=code_completion_request["test"],
            timeout=self.timeout
        )

        # save scores
        output.score["exec_result"] = result["exec_result"]
        output.score[self.name] = result["passed"]