from datasets import load_dataset

from minieval.datatypes import Instance, LMOutput, Response, Task, TaskConfig
from minieval.extract import math_latex


class AIME(Task):
    hf_path = "allenai/aime-2021-2025"
    subsets = [2022, 2023, 2024, 2025]

    def __init__(self, config: TaskConfig):
        self.config = config
        self._requests = None

    @property
    def requests(self):
        if self._requests is None:
            dataset = load_dataset(path=self.hf_path, split="train")
            requests = list(map(self._process_doc, dataset))

            # Only keep questions from a single year if specified
            if self.config.subset is not None:
                requests = [req for req in requests if req.metadata["year"] == self.config.subset]
            
            self._requests = requests
        return self._requests

    def _process_doc(self, doc):
        problem_from = doc.get("url").split("/")[-2]
        year = problem_from.split("_")[0]
        aime_number = "AIME_" + problem_from.split("_")[2]

        return Instance(
            question=doc["problem"],
            gold_completion=doc["solution"],
            solution=doc["answer"],
            metadata={
                "id": aime_number,
                "year": year,
            },
        )

    @classmethod
    def extract_answer(self, response: Response, generation: LMOutput) -> list[str]:
        return math_latex.extract_math_answer(generation.text)
