from datasets import load_dataset

from minieval.datatypes import Instance, LMOutput, Task, TaskConfig, Response
from minieval.extract.code import CodeExtractor


class HumanEval(Task):
    hf_path = "openai_humaneval"

    def __init__(self, config: TaskConfig):
        self.config = config
        self._requests = None

    @property
    def requests(self):
        if self._requests is None:
            dataset = load_dataset(path=self.hf_path, name=self.config.subset, split="test")
            self._requests = list(map(self._process_instance, dataset))
        return self._requests

    def _process_instance(self, doc: dict) -> Instance:
        query = doc["prompt"]

        query = "```python\n" + query

        unit_tests = doc["test"] + f"\ncheck({doc['entry_point']})"

        return Instance(
            question=query,
            gold_completion=doc["canonical_solution"] + "```",
            solution=unit_tests,
            metadata={
                "id": doc["task_id"],
                "entry_point": doc["entry_point"],
                "answer_prefix": doc["prompt"]
            }
        )

    @classmethod
    def extract_answer(self, response: Response, generation: LMOutput) -> list[str]:
        code = CodeExtractor.extract_code(generation)

        # For Humaneval, we follow the original paper setup by adding the prompt to the generated code completion
        # as the prompt may provide additional library imports needed for the code execution.
        full_code = response.input.metadata["answer_prefix"] + code
        
        return {
            "res_id": response.input.metadata["id"],
            "completion": full_code,
            "test": response.input.solution,
        }


class HumanEvalPlus(HumanEval):
    hf_path = "evalplus/humanevalplus"