from datasets import load_dataset

from minieval.datatypes import Instance, LMOutput, Response, Task, TaskConfig
from minieval.extract.math_latex import MathExtractor


class MinervaMath(Task):
    subsets = [
        "algebra",
        "counting_and_probability",
        "geometry",
        "intermediate_algebra",
        "number_theory",
        "prealgebra",
        "precalculus",
    ]
    hf_path = "EleutherAI/hendrycks_math"

    def __init__(self, config: TaskConfig):
        self.config = config
        self._requests = None

    @property
    def requests(self):
        if self._requests is None:
            dataset = load_dataset(path=self.hf_path, name=self.config.subset, split="test")
            self._requests = list(map(self._process_instance, dataset))
        return self._requests

    def _process_instance(self, doc: dict) -> Instance:
        solution = MathExtractor.extract_answer(doc["solution"])[0]  # get primary extracted answer

        return Instance(
            question=doc["problem"],
            gold_completion=doc["solution"],
            solution=solution,
            metadata={"level": doc.get("level"), "type": doc.get("type")},
        )

    @classmethod
    def extract_answer(self, response: Response, generation: LMOutput) -> list[str]:
        return MathExtractor.extract_answer(generation.text)


class Math500(MinervaMath):
    hf_path = "HuggingFaceH4/MATH-500"

    @property
    def requests(self):
        if self._requests is None:
            dataset = load_dataset(path=self.hf_path, split="test")
            self._requests = list(map(self._process_instance, dataset))
        return self._requests
